@@ -52,6 +52,10 @@
 #   define NV_LINUX
 #endif  /* defined(__linux__) */
 
+#if defined(__VMWARE__) && !defined(NV_VMWARE)
+#   define NV_VMWARE
+#endif /* defined(__VMWARE__) */
+
 /* SunOS + gcc */
 #if defined(__sun__) && defined(__svr4__) && !defined(NV_SUNOS)
 #   define NV_SUNOS
