@@ -26,7 +26,7 @@ static struct cdevsw nvidia_dev_cdevsw = {
     .d_mmap_single = nvidia_dev_mmap_single,
     .d_name =      "nvidia",
     .d_version =   D_VERSION,
-    .d_flags =     D_MEM|D_TRACKCLOSE
+    .d_flags =     D_MEM
 };
 
 static int nvidia_dev_open(
@@ -52,7 +52,7 @@ static int nvidia_dev_open(
     }
 
     filep->nv = nv;
-    mtx_init(&filep->event_mtx, "event_mtx", NULL, (MTX_SPIN | MTX_RECURSE));
+    mtx_init(&filep->event_mtx, "event_mtx", NULL, (MTX_DEF | MTX_RECURSE));
     STAILQ_INIT(&filep->event_queue);
 
     nv_lock_api(nv);
@@ -73,8 +73,6 @@ static void nvidia_dev_dtor(void *arg)
     status = nvidia_close_dev(nv, filep);
     nv_unlock_api(nv);
 
-    devfs_clear_cdevpriv();
-
     while ((et = STAILQ_FIRST(&filep->event_queue))) {
         STAILQ_REMOVE(&filep->event_queue, et, nvidia_event, queue);
         free(et, M_NVIDIA);
@@ -123,9 +121,9 @@ static int nvidia_dev_poll(
 
     status = devfs_get_cdevpriv((void **)&filep);
     if (status != 0)
-        return status;
+        return 0;
 
-    mtx_lock_spin(&filep->event_mtx);
+    mtx_lock(&filep->event_mtx);
     et = STAILQ_FIRST(&filep->event_queue);
 
     if (et == NULL)
@@ -133,7 +131,7 @@ static int nvidia_dev_poll(
     else
         mask = (events & (POLLIN | POLLPRI | POLLRDNORM));
 
-    mtx_unlock_spin(&filep->event_mtx);
+    mtx_unlock(&filep->event_mtx);
 
     return mask;
 }
