@@ -225,6 +225,25 @@ void NV_API_CALL os_unmap_kernel_space(
     pmap_unmapdev((vm_offset_t)address, size);
 }
 
+void* NV_API_CALL os_map_user_space(
+    NvU64   start,
+    NvU64   size_bytes,
+    NvU32   mode,
+    NvU32   protect,
+    void  **priv_data
+)
+{
+    return (void *)(NvUPtr)start;
+}
+
+void NV_API_CALL os_unmap_user_space(
+    void  *address,
+    NvU64  size,
+    void  *priv_data
+)
+{
+}
+
 RM_STATUS NV_API_CALL os_set_mem_range(
     NvU64 start,
     NvU64 size,
@@ -354,7 +373,7 @@ void NV_API_CALL out_string(const char *message)
     printf("%s", message);
 }
 
-#define MAX_ERROR_STRING 256
+#define MAX_ERROR_STRING 512
 static char nv_error_string[MAX_ERROR_STRING];
 
 int NV_API_CALL nv_printf(
@@ -369,7 +388,7 @@ int NV_API_CALL nv_printf(
 
     if (debuglevel >= ((cur_debuglevel >> 4) & 3)) {
         va_start(arglist, format);
-        chars_written = vsprintf(message, format, arglist);
+        chars_written = vsnprintf(message, sizeof(nv_error_string), format, arglist);
         va_end(arglist);
         printf("%s", message);
     }
@@ -593,7 +612,7 @@ RM_STATUS NV_API_CALL os_alloc_sema(void **semaphore)
     return RM_OK;
 }
 
-RM_STATUS NV_API_CALL os_free_sema(void *semaphore)
+void NV_API_CALL os_free_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
     nv_stack_t *sp;
@@ -605,8 +624,6 @@ RM_STATUS NV_API_CALL os_free_sema(void *semaphore)
     cv_destroy(&mtx->mutex_cv);
 
     os_free_mem(semaphore);
-
-    return RM_OK;
 }
 
 RM_STATUS NV_API_CALL os_acquire_sema(void *semaphore)
@@ -662,6 +679,137 @@ BOOL NV_API_CALL os_is_acquired_sema(void *semaphore)
     return (mtx->refcnt < 1);
 }
 
+
+RM_STATUS NV_API_CALL os_alloc_mutex(void **mutex)
+{
+    RM_STATUS status;
+    struct mtx *mtx;
+
+    status = os_alloc_mem((void **)&mtx, sizeof(struct mtx));
+    if (status != RM_OK)
+        return status;
+
+    mtx_init(mtx, "mtx", NULL, MTX_DEF | MTX_RECURSE);
+
+    *mutex = (void *)mtx;
+
+    return RM_OK;
+}
+
+void NV_API_CALL os_free_mutex(void *mutex)
+{
+    struct mtx *mtx = mutex;
+
+    if (mtx != NULL) {
+        mtx_assert(mtx, MA_OWNED);
+        mtx_destroy(mtx);
+        os_free_mem(mtx);
+    }
+}
+
+RM_STATUS NV_API_CALL os_acquire_mutex(void *mutex)
+{
+    struct mtx *mtx = mutex;
+
+    if (__NV_ITHREAD()) {
+        return RM_ERR_INVALID_REQUEST;
+    }
+
+    mtx_lock(mtx);
+    return RM_OK;
+}
+
+RM_STATUS NV_API_CALL os_cond_acquire_mutex(void *mutex)
+{
+    struct mtx *mtx = mutex;
+    if (__NV_ITHREAD()) {
+        return RM_ERR_INVALID_REQUEST;
+    }
+
+    if (mtx_trylock(mtx) == 0) {
+        return RM_ERR_TIMEOUT_RETRY;
+    }
+
+    return RM_OK;
+}
+
+void NV_API_CALL os_release_mutex(void *mutex)
+{
+    struct mtx *mtx = mutex;
+    mtx_unlock(mtx);
+}
+
+struct os_semaphore {
+    struct mtx mutex_mtx;
+    struct cv mutex_cv;
+    NvS32  count;
+    NvS32  limit;
+};
+
+
+void* NV_API_CALL os_alloc_semaphore(NvU32 initialValue, NvU32 limit)
+{
+    RM_STATUS status;
+    struct os_semaphore *os_sema;
+
+    status = os_alloc_mem((void **)&os_sema, sizeof(struct os_semaphore));
+    if (status != RM_OK) {
+        nv_printf(NV_DBG_ERRORS, "NVRM: failed to allocate semaphore!\n");
+        return NULL;
+    }
+
+    mtx_init(&os_sema->mutex_mtx, "os.sema_mtx", NULL, MTX_DEF | MTX_RECURSE);
+    cv_init(&os_sema->mutex_cv, "os.sema_cv");
+
+    os_sema->count = 1;
+    os_sema->limit = limit;
+
+    return (void *)os_sema;
+}
+
+void NV_API_CALL os_free_semaphore(void *semaphore)
+{
+    struct os_semaphore *os_sema = (struct os_semaphore *)semaphore;
+
+    mtx_destroy(&os_sema->mutex_mtx);
+    cv_destroy(&os_sema->mutex_cv);
+
+    os_free_mem(os_sema);
+}
+
+RM_STATUS NV_API_CALL os_acquire_semaphore(void *semaphore)
+{
+    struct os_semaphore *os_sema = (struct os_semaphore *)semaphore;
+
+    mtx_lock(&os_sema->mutex_mtx);
+    os_sema->count--;
+    if (os_sema->count < 0) {
+        cv_wait(&os_sema->mutex_cv, &os_sema->mutex_mtx);
+    }
+    mtx_unlock(&os_sema->mutex_mtx);
+
+    return RM_OK;
+}
+
+RM_STATUS NV_API_CALL os_release_semaphore(void *semaphore)
+{
+    struct os_semaphore *os_sema = (struct os_semaphore *)semaphore;
+
+    mtx_lock(&os_sema->mutex_mtx);
+    if (os_sema->count < 0) {
+        cv_signal(&os_sema->mutex_cv);
+    }
+    os_sema->count++;
+    mtx_unlock(&os_sema->mutex_mtx);
+
+    return RM_OK;
+}
+
+BOOL NV_API_CALL os_semaphore_may_sleep(void)
+{
+    return (!__NV_ITHREAD());
+}
+
 BOOL NV_API_CALL os_pat_supported(void)
 {
     return TRUE;
@@ -726,3 +874,8 @@ RM_STATUS NV_API_CALL os_free_smp_barrier(void)
 {
     return RM_OK;
 }
+
+RM_STATUS NV_API_CALL os_get_version_info(os_version_info * pOsVersionInfo)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
