@@ -162,8 +162,10 @@ void nvidia_pci_check_config_space(
             if (BAR->offset == 0) continue;
 
             BAR_low = os_pci_read_dword(dev, BAR->offset);
-            if ((BAR_low & NVRM_PCICFG_BAR_ADDR_MASK) != BAR->address)
-                os_pci_write_dword(dev, BAR->offset, BAR->address);
+            if ((BAR_low & NVRM_PCICFG_BAR_ADDR_MASK) != 
+                (BAR->address & 0xffffffff))
+                os_pci_write_dword(dev, BAR->offset, 
+                                   (NvU32)(BAR->address & 0xffffffff));
 
             if ((BAR_low & NVRM_PCICFG_BAR_MEMTYPE_MASK)
                     != NVRM_PCICFG_BAR_MEMTYPE_64BIT)
@@ -174,8 +176,8 @@ void nvidia_pci_check_config_space(
                     && BAR_high == 0)
                 continue;
 
-            os_pci_write_dword(dev, BAR->offset, BAR->address); /* 31:4 */
-            os_pci_write_dword(dev, BAR->offset + 4, 0); /* 63:32 */
+            if (BAR_high != (BAR->address >> 32))
+                os_pci_write_dword(dev, BAR->offset + 4, (BAR->address >> 32));
         }
     }
 }
@@ -218,6 +220,7 @@ int nvidia_pci_attach(device_t dev)
     struct nvidia_softc *sc;
     NvU16 word, i, j;
     NvU32 BAR_low, req;
+    NvU64 BAR_high;
     nv_stack_t *sp;
 
     if (device_get_unit(dev) >= NV_MAX_DEVICES) {
@@ -256,9 +259,14 @@ int nvidia_pci_attach(device_t dev)
         if ((req != 0) /* implemented */ && (req & NVRM_PCICFG_BAR_REQTYPE_MASK)
                 == NVRM_PCICFG_BAR_REQTYPE_MEMORY) {
             sc->nv_state->bars[j].offset = offset;
-            sc->BAR_rids[j] = offset; j++;
+            sc->BAR_rids[j] = offset; 
             if ((req & NVRM_PCICFG_BAR_MEMTYPE_MASK) == NVRM_PCICFG_BAR_MEMTYPE_64BIT)
+            {
+                BAR_high = os_pci_read_dword(dev, offset + 4);
+                sc->nv_state->bars[j].address |= (BAR_high << 32);
                 i++;
+            }
+            j++;
         }
         os_pci_write_dword(dev, offset, BAR_low);
     }
