@@ -175,12 +175,12 @@ typedef NvU32           NvLength; /* length to agree with sizeof             */
 
 // XXX Obsolete -- get rid of me...
 typedef NvP64 NvP64_VALUE_T;
-typedef NvUPtr NV_UINTPTR_T;
 #define NvP64_LVALUE(n)   (n)
 #define NvP64_SELECTOR(n) (0)
 
 /* Useful macro to hide required double cast */
-#define NV_PTR_TO_NvP64(n) (NvP64)(NV_UINTPTR_T)(n)
+#define NV_PTR_TO_NvP64(n) (NvP64)(NvUPtr)(n)
+#define NV_SIGN_EXT_PTR_TO_NvP64(p) ((NvP64)(NvS64)(NvSPtr)(p))
 #define KERNEL_POINTER_TO_NvP64(p) ((NvP64)(uintptr_t)(p))
 
 /* obsolete stuff  */
@@ -271,6 +271,12 @@ typedef unsigned char *PUCHAR;
 #define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) __align(ALIGN) TYPE_VAR
 #endif
 
+// NVRM_IMPORT is defined on windows for nvrm4x build (nvrm4x.lib).
+#if (defined(NV_WINDOWS) && defined(NVRM4X_BUILD))
+#define NVRM_IMPORT __declspec(dllimport)
+#else
+#define NVRM_IMPORT
+#endif
 
  /***************************************************************************\
 |*                       Function Declaration Types                          *|
