@@ -50,6 +50,15 @@
 #endif
 #endif /* !defined(NV_API_CALL) */
 
+typedef struct
+{
+    NvU32  os_major_version;
+    NvU32  os_minor_version;
+    NvU32  os_build_number;
+    char * os_build_version_str;
+    char * os_build_date_plus_str;
+}os_version_info;
+
 /*
  * ---------------------------------------------------------------------------
  *
@@ -88,6 +97,8 @@ void        NV_API_CALL  os_pci_write_word           (void *, NvU8, NvU16);
 void        NV_API_CALL  os_pci_write_dword          (void *, NvU8, NvU32);
 void*       NV_API_CALL  os_map_kernel_space         (NvU64, NvU64, NvU32);
 void        NV_API_CALL  os_unmap_kernel_space       (void *, NvU64);
+void*       NV_API_CALL  os_map_user_space           (NvU64, NvU64, NvU32, NvU32, void **);
+void        NV_API_CALL  os_unmap_user_space         (void *, NvU64, void *);
 RM_STATUS   NV_API_CALL  os_flush_cpu_cache          (void);
 void        NV_API_CALL  os_flush_cpu_write_combine_buffer(void);
 RM_STATUS   NV_API_CALL  os_set_mem_range            (NvU64, NvU64, NvU32);
@@ -114,7 +125,7 @@ RM_STATUS   NV_API_CALL  os_disable_console_access   (void);
 RM_STATUS   NV_API_CALL  os_enable_console_access    (void);
 RM_STATUS   NV_API_CALL  os_registry_init            (void);
 RM_STATUS   NV_API_CALL  os_alloc_sema               (void **);
-RM_STATUS   NV_API_CALL  os_free_sema                (void *);
+void        NV_API_CALL  os_free_sema                (void *);
 RM_STATUS   NV_API_CALL  os_acquire_sema             (void *);
 RM_STATUS   NV_API_CALL  os_cond_acquire_sema        (void *);
 RM_STATUS   NV_API_CALL  os_release_sema             (void *);
@@ -127,6 +138,17 @@ RM_STATUS   NV_API_CALL  os_queue_work_item          (void *);
 RM_STATUS   NV_API_CALL  os_flush_work_queue         (void);
 void        NV_API_CALL  os_register_compatible_ioctl    (NvU32, NvU32);
 void        NV_API_CALL  os_unregister_compatible_ioctl  (NvU32, NvU32);
+RM_STATUS   NV_API_CALL  os_alloc_mutex              (void **);
+void        NV_API_CALL  os_free_mutex               (void *);
+RM_STATUS   NV_API_CALL  os_acquire_mutex            (void *);
+RM_STATUS   NV_API_CALL  os_cond_acquire_mutex       (void *);
+void        NV_API_CALL  os_release_mutex            (void *);
+void*       NV_API_CALL  os_alloc_semaphore          (NvU32, NvU32);
+void        NV_API_CALL  os_free_semaphore           (void *);
+RM_STATUS   NV_API_CALL  os_acquire_semaphore        (void *);
+RM_STATUS   NV_API_CALL  os_release_semaphore        (void *);
+BOOL        NV_API_CALL  os_semaphore_may_sleep      (void);
+RM_STATUS   NV_API_CALL  os_get_version_info         (os_version_info*);
 
 /* BOOL os_pat_supported(void) 
  * report to core resman whether pat is supported for marking cache attributes
@@ -185,12 +207,16 @@ void NV_API_CALL  nv_prints(NvU32 debuglevel, const char *string);
 #define NV_MEMORY_CACHED            0
 #define NV_MEMORY_UNCACHED          1
 #define NV_MEMORY_WRITECOMBINED     2
-/* #define NV_MEMORY_WRITETHRU         3 */
-/* #define NV_MEMORY_WRITEPROTECT      4 */
-/* #define NV_MEMORY_WRITEBACK         5 */
+#define NV_MEMORY_WRITETHRU         3
+#define NV_MEMORY_WRITEPROTECT      4
+#define NV_MEMORY_WRITEBACK         5
 #define NV_MEMORY_DEFAULT           6
 #define NV_MEMORY_UNCACHED_WEAK     7
 
+#define NV_PROTECT_READABLE   1
+#define NV_PROTECT_WRITEABLE  2
+#define NV_PROTECT_READ_WRITE (NV_PROTECT_READABLE | NV_PROTECT_WRITEABLE)
+
 /* in some cases, the os may have a different page size, but the
  * hardware (fb, regs, etc) still address and "think" in 4k
  * pages. make sure we can mask and twiddle with these addresses when
