@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1993-2008 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1993-2011 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -99,10 +99,11 @@ typedef NvU32 RM_STATUS;
 #define RM_ERR_INVALID_LOCK_STATE       0x00000047
 #define RM_ERR_INSUFFICIENT_POWER       0x00000048
 #define RM_ERR_REJECTED_VBIOS           0x00000049
-#define RM_ERR_MEMORY_TRAINING_FAILED   0X0000004A
+#define RM_ERR_MEMORY_TRAINING_FAILED   0x0000004A
+#define RM_ERR_COMPRESSION_ERROR        0x0000004B
 #define RM_ERR_I2C_SPEED_TOO_HIGH       0x00000050
 #define RM_ERR_INVALID_IRQ_LEVEL        0x00000051
-#define RM_ERR_BUSY_RETRY               0X00000052
+#define RM_ERR_BUSY_RETRY               0x00000052
 
 // Warnings:
 #define RM_WARN_NULL_OBJECT             0x00000100
